import datetime

import plotly.figure_factory as ff
import plotly.graph_objects as go
import streamlit as st


class DataHealth:
    def __init__(self, mlfoundry_data):
        self.mlfoundry_data = mlfoundry_data

    def start_tab(self, project_name, id, run_ids):

        ## Whylogs profile summary computation
        profile_summary = self.mlfoundry_data.get_whylogs_summary(
            st.session_state["id_model"]
        )

        if profile_summary is None:
            st.warning(
                f"Profile summary doesn't exists for run {st.session_state['id_model']}"
            )
            return
        summary = profile_summary["summary"]
        summary = summary.fillna("null")

        st.subheader("Profile Summary")

        st.dataframe(summary)



import logging

import streamlit as st

from mlfoundry_ui.webapp.dashboard_constants import *

logger = logging.getLogger(__name__)


class SideBar:
    def __init__(self, mlfoundry_data):
        self.mlfoundry_data = mlfoundry_data

    @staticmethod
    def project_name_callback(name, page):
        if page == Pages.SINGLE_RUN_VIEW.value:
            st.session_state[PROJECT_SESSION_STATE_MODEL_VIEW] = name
        elif page == Pages.RUN_COMPARISON.value:
            st.session_state[PROJECT_SESSION_STATE_MODEL_COMPARISON] = name

    @staticmethod
    def run_id_callback(run_id, run_name):
        st.session_state[RUN_NAME_SESSION_STATE_MODEL_VIEW] = run_name
        st.session_state[ID_SESSION_STATE_MODEL_VIEW] = run_id

    def sidebar(self, page, project_name_to_id_dict):
        """

        :param page:
        :param project_name_to_id_dict:
        :return:
        """
        if not project_name_to_id_dict:
            st.sidebar.error(f"No project names provided to show data for.")
            return
        project_names = [i for i in project_name_to_id_dict.keys()]
        with st.sidebar.form(f"Project Name {page}"):
            project_name = st.selectbox("Project Name", options=project_names)
            submit_button = st.form_submit_button(
                "Submit",
                on_click=self.project_name_callback,
                args=[project_name, page],
            )
        if page == Pages.SINGLE_RUN_VIEW.value:  # Is this is a single run view
            if PROJECT_SESSION_STATE_MODEL_VIEW not in st.session_state:
                return
            with st.sidebar.form("run_id_form"):
                with st.spinner("Loading Run IDs"):
                    project_id = project_name_to_id_dict[project_name]
                    try:
                        run_name_to_id_dict = self.mlfoundry_data.get_runs_in_project(
                            project_id
                        )
                    except Exception as e:
                        err_msg = f"Was not able to fetch any runs for project {project_name}. Error {e}"
                        st.sidebar.warning(err_msg)
                        logger.exception(err_msg)
                        return
                if not run_name_to_id_dict:
                    err_msg = (
                        f"Was not able to fetch any runs for project {project_name}."
                    )
                    st.sidebar.warning(err_msg)
                    logger.error(err_msg)
                    return
                run_name = st.selectbox(
                    "Run Name",
                    list([run_name for run_name in run_name_to_id_dict.keys()]),
                )
                run_id = run_name_to_id_dict[run_name]

                id_submit_button = st.form_submit_button(
                    "Submit", on_click=self.run_id_callback, args=[run_id, run_name]
                )

            return run_name_to_id_dict

        elif page == Pages.RUN_COMPARISON.value:  # Is this is a run comparison view

            if PROJECT_SESSION_STATE_MODEL_COMPARISON not in st.session_state:
                return
            project_id = project_name_to_id_dict[project_name]
            try:
                run_name_to_id_dict = self.mlfoundry_data.get_runs_in_project(
                    project_id
                )
            except Exception as e:
                err_msg = f"Was not able to fetch any runs for project {project_name}. Error {e}"
                st.sidebar.warning(err_msg)
                logger.error(err_msg)
                return
            if not run_name_to_id_dict:
                err_msg = f"Was not able to fetch any runs for project {project_name}."
                st.sidebar.warning(err_msg)
                logger.error(err_msg)
                return
            return run_name_to_id_dict
        else:
            logger.error(
                f"Received page value {page} but expecting one of {[e.value for e in Pages]}"
            )



import datetime
import logging
import os
from typing import Optional

import mlflow
import pandas as pd
import streamlit as st
import whylogs
from mlflow.entities import ViewType
from mlflow.tracking import MlflowClient

from mlfoundry_ui.constants import MLRUNS_FOLDER_NAME
from mlfoundry_ui.webapp.dashboard_constants import *

logger = logging.getLogger(__name__)


class MLFoundryData:
    def __init__(self, mlruns_path: Optional[str] = None):
        # TODO (frizwankhan): Add support for remote.
        if mlruns_path is None:
            mlruns_path = os.path.abspath(MLRUNS_FOLDER_NAME)
        if not os.path.exists(mlruns_path):
            raise Exception(f"{mlruns_path} does not exist.")
        # TODO (nikunjbjj): Figure out if this is the cleanest solution we have.
        # MLFlow requires the URI to start with file. This might need to change when we add support for remote.
        if not mlruns_path.startswith("file:"):
            mlruns_path = "file:" + mlruns_path
        self.mlruns_path = mlruns_path
        mlflow.set_tracking_uri(mlruns_path)
        self.mlflow_client = MlflowClient()

    def get_runs_in_project(self, project_id):
        try:
            all_runs = self.mlflow_client.list_run_infos(
                project_id, run_view_type=ViewType.ALL
            )
        except Exception as e:
            err_msg = f"Problem happened fetching runs for project id {project_id}. Error is {e}"
            st.warning(err_msg)
            logger.exception(err_msg)
            return

        run_name_to_id_dict = {}
        for run in all_runs:
            try:
                # TODO (niikunjbjj): Check if MLFlow has an API to fetch run tags
                with mlflow.start_run(
                    run_id=run.run_id, experiment_id=project_id
                ) as run1:
                    run1_tags = run1.data.tags
                run_name = run1_tags.get(RUN_NAME_TAG, "")
                if run_name == "":
                    logger.warning(f"Couldn't find run_name for run id {run.run_id}")
                    continue
                run_name_to_id_dict[run_name] = run.run_id
            except Exception as e:
                logger.warning(
                    f"Problem fetching detail of run with run_id {run.run_id}. Skipping. Error {e}"
                )
        return run_name_to_id_dict

    @st.experimental_memo(suppress_st_warning=True)
    def get_model_type(_self, run_id):
        # TODO(Rizwan): streamlit doesnt know how to hash self,
        #  so to specify it to not hash it we are adding a underscore before it
        # Have to find a proper solution to hash self
        with mlflow.start_run(run_id=run_id) as run1:
            run1_tags = run1.data.tags

        return run1_tags.get(MODEL_TYPE_TAG)

    def project_options(self):
        try:
            experiments = self.mlflow_client.list_experiments(view_type=ViewType.ALL)
        except Exception as e:
            err_msg = f"Could not fetch logged experiments. Error {e}"
            st.warning(err_msg)
            logger.exception(err_msg)
            return

        project_name_to_id_dict = {}
        for exp in experiments:
            try:
                if exp.experiment_id == "0":  # Removing the default e
                    continue
                project_name_to_id_dict[exp.name] = exp.experiment_id
            except Exception as e:
                logger.warning(
                    f"Problem fetching detail of run with experiment id {exp.experiment_id} and name {exp.name}. "
                    f"Skipping. Error {e}"
                )
        return project_name_to_id_dict

    @st.experimental_memo(suppress_st_warning=True)
    def get_metrics(_self, run_ids):
        # TODO(Rizwan): streamlit doesnt know how to hash self, so to specify it to not hash
        #  it we are adding a underscore before it
        # Have to find a proper solution to hash self
        metrics = []
        for run_id in run_ids:
            run = _self.mlflow_client.get_run(run_id)
            run_name = run.data.tags[RUN_NAME_TAG]
            run_metrics = run.data.metrics

            for metric in run_metrics.keys():
                metric_list = []
                for metric_item in _self.mlflow_client.get_metric_history(
                    run_id, metric
                ):
                    metric_list.append(metric_item.value)
                if len(metric_list) < 2:
                    metric_list = metric_list[0]
                metrics.append([metric, metric_list, run_id, run_name])
        return pd.DataFrame(
            metrics,
            columns=[
                METRIC_DF_KEY_COL,
                METRIC_DF_VALUE_COL,
                METRIC_DF_RUNID_COL,
                METRIC_DF_RUN_NAME_COL,
            ],
        )

    @st.experimental_memo(suppress_st_warning=True)
    def get_artifact(_self, run_id, artifact_name):
        # TODO(Rizwan): streamlit doesnt know how to hash self, so to specify it to not hash
        #  it we are adding a underscore before it
        # Have to find a proper solution to hash self
        try:
            return _self.mlflow_client.download_artifacts(run_id, artifact_name)
        except Exception as e:
            err_msg = f"Artifact {artifact_name} not found. Exception: {e}"
            st.warning(err_msg)
            logger.exception(err_msg)
            return

    @st.experimental_memo(suppress_st_warning=True)
    def get_whylogs_summary(_self, run_id):
        # TODO(Rizwan): streamlit doesnt know how to hash self, so to specify it to not hash
        #  it we are adding a underscore before it
        # Have to find a proper solution to hash self

        dir_path = _self.get_artifact(run_id, WHYLOGS_ARTIFACT_DIR)
        if dir_path is None:
            return

        if len(os.listdir(dir_path)) == 0:
            return None

        whylogs_session = whylogs.get_or_create_session()
        profiles = []
        for file in os.listdir(dir_path):
            profile = whylogs_session.new_profile(
                dataset_name="in-memory", dataset_timestamp=datetime.datetime.now()
            )
            profile = profile.read_protobuf(os.path.join(dir_path, file))
            profiles.append(profile)

        # merging profiles
        profile = profiles[0]
        for i in range(1, len(profiles)):
            profile = profile.merge(profiles[i])

        profile_summary = profile.flat_summary()
        return profile_summary

    def get_mlruns_path(self):
        return self.mlruns_path



import logging
import random

import plotly.graph_objects as go
import streamlit as st

from mlfoundry_ui.webapp.dashboard_constants import *

# from scipy.sparse.construct import random


logger = logging.getLogger(__name__)


class ModelComparison:
    def __init__(self, mlfoundry_data) -> None:
        self.mlfoundry_data = mlfoundry_data

    def start_tab(self, run_ids):
        metrics_df = self.mlfoundry_data.get_metrics(run_ids)
        metrics_df_list = metrics_df[
            metrics_df[METRIC_DF_VALUE_COL].apply(lambda x: isinstance(x, list))
        ]
        metrics_df = metrics_df[
            ~metrics_df[METRIC_DF_VALUE_COL].apply(lambda x: isinstance(x, list))
        ]

        comparison_columns = st.selectbox(
            "Comparison By", [METRIC_DF_RUNID_COL, METRIC_DF_RUN_NAME_COL], 0
        )

        unique_runs = list(
            set(
                list(metrics_df[comparison_columns])
                + list(metrics_df_list[comparison_columns])
            )
        )
        if comparison_columns == METRIC_DF_RUNID_COL:
            runs = st.multiselect("Run IDs", unique_runs, unique_runs)
        elif comparison_columns == METRIC_DF_RUN_NAME_COL:
            runs = st.multiselect("Run Names", unique_runs, unique_runs)
        else:
            logger.error(
                f"Expecting one of the following "
                f"{METRIC_DF_RUNID_COL, METRIC_DF_RUN_NAME_COL} to be selected. Got {comparison_columns}"
            )
            return

        if len(metrics_df) == 0:
            logger.info(f"No metrics are logged.")
            return

        metrics = list(metrics_df[METRIC_DF_KEY_COL].unique())
        metrics_df = metrics_df[metrics_df[comparison_columns].isin(runs)]
        dat = []
        for metric in metrics:
            try:
                x = list(
                    metrics_df[metrics_df[METRIC_DF_KEY_COL] == metric][
                        comparison_columns
                    ]
                )
                y = list(
                    metrics_df[metrics_df[METRIC_DF_KEY_COL] == metric][
                        METRIC_DF_VALUE_COL
                    ]
                )
                dat.append(go.Bar(x=x, y=y, name=metric))
                fig = go.Figure(data=[go.Bar(x=x, y=y)])
                fig.update_traces(
                    marker_color=random.choice(COLORS),
                    marker_line_width=1.5,
                    opacity=0.6,
                )
                fig.update_layout(title_text=metric)
                st.write(fig)
            except Exception as err:
                logger.error(
                    f"Error happened in plotting metric {metric}. Error: {err}"
                )

        fig = go.Figure(data=dat)
        fig.update_layout(barmode="group", width=1000, height=600)
        st.write(fig)



import logging
import sys

import hydralit as hy
import streamlit as st

from mlfoundry_ui.constants import MLFOUNDRY_DOCUMENTATION_LINK
from mlfoundry_ui.webapp.dashboard_constants import *
from mlfoundry_ui.webapp.mlfoundry_data import MLFoundryData
from mlfoundry_ui.webapp.model_comparison.model_comparison import ModelComparison
from mlfoundry_ui.webapp.model_view.data_health import DataHealth
from mlfoundry_ui.webapp.model_view.feature_view import FeatureView
from mlfoundry_ui.webapp.model_view.model_health import ModelHealth
from mlfoundry_ui.webapp.sidebar import SideBar

logger = logging.getLogger(__name__)


def page_navigation():
    with st.sidebar:
        selected_page = st.radio("Page", [e.value for e in Pages])
    return selected_page


def deal_with_single_run_view(run_name_to_id_dict, mlfoundry_data, app):
    if ID_SESSION_STATE_MODEL_VIEW in st.session_state:
        run_ids = [run_id for run_id in run_name_to_id_dict.values()]

        @app.addapp(title="Model Health")
        def model_health():
            modelhealth_page = ModelHealth(mlfoundry_data)
            modelhealth_page.start_tab(
                st.session_state[PROJECT_SESSION_STATE_MODEL_VIEW], run_ids
            )

        @app.addapp(title="Data Health")
        def data_health():
            datahealth_page = DataHealth(mlfoundry_data)
            datahealth_page.start_tab(
                st.session_state[PROJECT_SESSION_STATE_MODEL_VIEW],
                st.session_state[ID_SESSION_STATE_MODEL_VIEW],
                run_ids,
            )

        @app.addapp(title="Feature Health")
        def feature_view():
            featureview_page = FeatureView(mlfoundry_data)
            featureview_page.start_tab(
                st.session_state[PROJECT_SESSION_STATE_MODEL_VIEW],
                st.session_state[ID_SESSION_STATE_MODEL_VIEW],
                run_ids,
            )

        app.run()


def deal_with_model_comparison_view(run_name_to_id_dict, mlfoundry_data, app):
    if PROJECT_SESSION_STATE_MODEL_COMPARISON in st.session_state:
        run_ids = [run_id for run_id in run_name_to_id_dict.values()]

        @app.addapp(title="Model Comparison")
        def comparison():
            modelcomparison_page = ModelComparison(mlfoundry_data)
            modelcomparison_page.start_tab(run_ids)

        app.run()


def main():

    # TODO (nikunjbjj): Need to use argparse here.
    if len(sys.argv) > 1:
        mlruns_path = sys.argv[1]
    else:
        mlruns_path = ""

    app = hy.HydraApp(
        title="MLFoundry dashboard",
        hide_streamlit_markers=False,
        navbar_sticky=True,
    )
    mlfoundry_data = MLFoundryData(mlruns_path)
    sidebar = SideBar(mlfoundry_data)

    project_name_to_id_dict = mlfoundry_data.project_options()
    if not project_name_to_id_dict:
        st.warning(
            f"No projects are logged in the path {mlruns_path} using mlfoundry. "
            f"Please use {MLFOUNDRY_DOCUMENTATION_LINK} to find instructions to log data."
            f"Can't bring up the dashboard."
        )
        return

    selected_page = page_navigation()

    run_name_to_id_dict = sidebar.sidebar(selected_page, project_name_to_id_dict)

    # TODO (nikunjbjj): Handle the case if the above function returns None because of error.
    # if run_ids_names_tuple is None:
    #     st.warning(
    #         f"Problem happened selecting the project or run. Can't bring up the dashboard"
    #     )
    #     return
    if selected_page == Pages.SINGLE_RUN_VIEW.value:
        deal_with_single_run_view(run_name_to_id_dict, mlfoundry_data, app)
    elif selected_page == Pages.RUN_COMPARISON.value:
        deal_with_model_comparison_view(run_name_to_id_dict, mlfoundry_data, app)
    else:
        logger.error(
            f"Received page value {selected_page} but expecting one of {[e.value for e in Pages]}"
        )


if __name__ == "__main__":
    main()



import json
import os
import pickle

import numpy as np
import pandas as pd
import plotly.express as px
import plotly.figure_factory as ff
import plotly.graph_objects as go
import streamlit as st

from mlfoundry_ui.webapp.dashboard_constants import *


class FeatureView:
    def __init__(self, dataclass):
        self.mlfoundry_data = dataclass

    def get_schema_shap(self):
        """
        Return the shap and schema corresponding to each dataslice in a dictionary
        """
        dir_path = self.mlfoundry_data.get_artifact(
            st.session_state[ID_SESSION_STATE_MODEL_VIEW], "stats"
        )
        if dir_path is None:
            st.warning(
                f"Feature Importance doesn't exists for run {st.session_state['id_model']}"
            )
            return None

        dataslice_schemas = {}
        dataslice_shap = {}
        for file_name in os.listdir(dir_path):
            file_path = os.path.join(dir_path, file_name)
            for data_slice_name in DATA_SLICES:
                if data_slice_name in file_name and SCHEMA_FILE_NAME in file_name:
                    with open(file_path, "rb") as f:
                        schema = json.load(f)
                    dataslice_schemas[data_slice_name] = schema

                elif SHAP_VALUE_FILE_NAME in file_name and data_slice_name in file_name:
                    with open(file_path, "rb") as f:
                        shap_values = pickle.load(f)

                    shaps = []
                    num_class = len(shap_values)
                    for ind in range(num_class):
                        global_shap_values = np.abs(shap_values[ind]).mean(0)
                        shaps.append(global_shap_values)
                    df = pd.DataFrame(
                        np.transpose(np.array(shaps)), columns=list(range(num_class))
                    )
                    dataslice_shap[data_slice_name] = df

        return dir_path, dataslice_schemas, dataslice_shap

    # @st.experimental_memo
    def shap_computation(self, dataslice_schemas, dataslice_shap):
        """
        Shap computation given the dataslice_schemas and dataslice_shap
        """
        cols = list(st.columns(2))
        if len(dataslice_shap) > 2:
            cols += list(st.columns(2))

        for i, (data_slice_name, shap_df) in enumerate(dataslice_shap.items()):
            if data_slice_name not in dataslice_schemas.keys():
                continue
            # for each dataslice getting thre shap corresponding to the dataslice
            shap_df["Features"] = dataslice_schemas[data_slice_name][
                "feature_column_names"
            ]
            shap_df = shap_df.sort_values(shap_df.columns[0])
            with cols[i]:
                st.subheader(f"Feature Importance - {data_slice_name} data")
                num_class = len(dataslice_schemas[data_slice_name])
                feature_cols = list(shap_df.columns)
                feature_cols.remove("Features")
                fig = px.bar(shap_df, y="Features", x=feature_cols, orientation="h")
                fig.update_layout(
                    xaxis_title="mean(|SHAP value|)(average impact on model output magnitude)"
                )
                st.write(fig)

    def actual_vs_preds_classification(self, dir_path, dataslice_schemas):
        """
        unique counts file example:
        {'preds': (array([0, 1]), array([1574,   93])),
        'actuals': (array([0, 1]), array([1443,  224])),
        'state_AZ': (array([0, 1]), array([1642,   25]))
        }
        Each feature contains two array, first array conrresponds to unique values in the feature,
        second array correspond to counts corresponding to the unique values
        """
        data = []
        for data_slice_name in dataslice_schemas.keys():
            for file_name in os.listdir(dir_path):
                if (
                    data_slice_name not in file_name
                    or UNIQUE_COUNTS_FILE_NAME not in file_name
                ):
                    continue
                with open(os.path.join(dir_path, file_name), "rb") as f:
                    unique_counts = pickle.load(f)
                # getting the prediction and actual column name
                preds_col = dataslice_schemas[data_slice_name][
                    "prediction_column_name"
                ]  # getting the prediction and actual column name
                actual_col = dataslice_schemas[data_slice_name]["actual_column_name"]
                # getting the classes available in actual column
                classes = unique_counts[actual_col][0]
                ## For each dataslice finding the counts corresponding to classes
                for i in range(len(unique_counts[actual_col][1])):
                    data.append(
                        [
                            f"Class {unique_counts[actual_col][0][i]}",  # class number,
                            unique_counts[actual_col][1][
                                i
                            ],  # count corresponding to the class
                            "Truth Label",
                            data_slice_name,
                        ]
                    )
                for i in range(len(unique_counts[preds_col][1])):
                    data.append(
                        [
                            f"Class {unique_counts[preds_col][0][i]}",  # class number,
                            unique_counts[preds_col][1][
                                i
                            ],  # count corresponding to the class
                            "Predictions",
                            data_slice_name,
                        ]
                    )
        if len(data) != 0:
            pred_actual_df = pd.DataFrame(
                data, columns=["class", "count", "actual_preds", "data_slice"]
            )
            fig = px.bar(
                pred_actual_df,
                x="actual_preds",
                y="count",
                color="class",
                barmode="group",
                facet_row="data_slice",
            )
            fig.for_each_annotation(lambda a: a.update(text=a.text.split("=")[-1]))
            fig.update_annotations(font=dict(size=20))
            fig.update_layout(showlegend=True, xaxis_title="")
            st.subheader("Predictions and Truth Label distribution")
            st.write(fig)

    def whylogs_histogram(self, histogram):
        """
        Preprocesses the histogram data obtained from whylogs
        """
        values = []
        for i in range(len(histogram["bin_edges"]) - 1):
            value = (histogram["bin_edges"][i + 1] + histogram["bin_edges"][i]) / 2
            values += [value] * histogram["counts"][i]
        return values

    def actual_vs_preds_regression(self, dir_path, dataslice_schemas):

        for data_slice_name in dataslice_schemas.keys():
            for file_name in os.listdir(dir_path):
                if (
                    data_slice_name not in file_name
                    or UNIQUE_COUNTS_FILE_NAME not in file_name
                ):
                    continue
                with open(os.path.join(dir_path, file_name), "rb") as f:
                    unique_counts = pickle.load(f)
                preds_col = dataslice_schemas[data_slice_name][
                    "prediction_column_name"
                ]  # getting the prediction and actual column name
                actual_col = dataslice_schemas[data_slice_name]["actual_column_name"]

                actual_histogram = unique_counts[ACTUAL_PREDICTION_REGRESSION][
                    actual_col
                ]
                prediction_histogram = unique_counts[ACTUAL_PREDICTION_REGRESSION][
                    preds_col
                ]
                actual_values = self.whylogs_histogram(actual_histogram)
                prediction_values = self.whylogs_histogram(prediction_histogram)

                st.subheader(f"Actuals vs Prediction - {data_slice_name} data")
                fig = go.Figure()
                fig.add_trace(go.Histogram(x=actual_values, name=actual_col))
                fig.add_trace(go.Histogram(x=prediction_values, name=preds_col))
                fig.update_layout(barmode="overlay")
                fig.update_traces(opacity=0.6)
                st.write(fig)

    def categorical_feature_distribution(self, dir_path, features, dataslice_schemas):
        """
        unique counts file example:
        {'preds': (array([0, 1]), array([1574,   93])),
        'actuals': (array([0, 1]), array([1443,  224])),
        'state_AZ': (array([0, 1]), array([1642,   25]))
        }
        Each feature contains two array, first array conrresponds to unique values in the feature,
        second array correspond to counts corresponding to the unique values

        Returns:
        numerical_feature_names
        """
        numerical_features = features
        for key in dataslice_schemas.keys():
            data_slice_name = key
            categorical_features = dataslice_schemas[key][
                "categorical_feature_column_names"
            ]
            if categorical_features is not None:
                numerical_features = list(set(features) - set(categorical_features))
                break
        if categorical_features is None:
            return numerical_features

        st.subheader("Categorcal feature distribution")
        cat_feature_option = st.selectbox(
            "Categorical Feature", categorical_features, 0
        )

        for file in os.listdir(dir_path):
            if data_slice_name in file and UNIQUE_COUNTS_FILE_NAME in file:
                with open(os.path.join(dir_path, file), "rb") as f:
                    unique_counts = pickle.load(f)
                # Getting the unique count
                unique_count_feature = unique_counts[cat_feature_option]
                # unique_count_feature[0] corresponds to the unique values
                # unique_count_feature[1] correpons to the counts of the value
                fig = go.Figure(
                    [
                        go.Bar(
                            x=unique_count_feature[0],
                            y=unique_count_feature[1],
                            name=cat_feature_option,
                        )
                    ]
                )
                fig.update_layout(showlegend=True)
                st.write(fig)

        return numerical_features

    def numerical_feature_distribution(self, numerical_features, summary_hist):
        """
        Numercial feature distribution is obtained from whyogs
        """
        st.subheader("Numerical feature distribution")
        with st.form("features availabe"):
            feature_options = st.multiselect(
                "Features", numerical_features, [numerical_features[0]]
            )
            submit_button = st.form_submit_button(label="Submit")

        if len(feature_options) == 0:
            st.warning("Please select a feature")
            return

        if not submit_button:
            return
        values_full = []
        bin_sizes = []
        for feature in feature_options:
            values = []
            for i in range(len(summary_hist[feature][WHYLOGS_BINEDGES]) - 1):
                value = (
                    summary_hist[feature][WHYLOGS_BINEDGES][i]
                    + summary_hist[feature][WHYLOGS_BINEDGES][i + 1]
                ) / 2
                values = values + [value] * summary_hist[feature][WHYLOGS_COUNTS][i]
                if i == 0:
                    bin_size = (
                        summary_hist[feature][WHYLOGS_BINEDGES][i + 1]
                        - summary_hist[feature][WHYLOGS_BINEDGES][i]
                    )
                    bin_sizes.append(bin_size)
            values_full = values_full + [values]
        fig = ff.create_distplot(
            values_full, group_labels=feature_options, bin_size=bin_sizes
        )
        fig.update_layout(width=1000, height=650)
        st.write(fig)

    def actual_vs_prediction_time_series(self, dir_path, dataslice_schemas):
        """
        Actual vs Prediction is plotted for the timeseries dataset
        """
        for data_slice_name in dataslice_schemas.keys():
            for file_name in os.listdir(dir_path):
                if (
                    data_slice_name not in file_name
                    or TIME_SERIES_ACTUAL_PREDS_FILE_NAME not in file_name
                ):
                    continue
                preds_vs_actual = pd.read_parquet(os.path.join(dir_path, file_name))

                preds_col = dataslice_schemas[data_slice_name][
                    "prediction_column_name"
                ]  # getting the prediction and actual column name
                actual_col = dataslice_schemas[data_slice_name]["actual_column_name"]
                predictions = preds_vs_actual[preds_col]
                actuals = preds_vs_actual[actual_col]

                st.subheader(f"Actuals vs Prediction - {data_slice_name} data")
                fig = go.Figure()
                fig.add_trace(
                    go.Scatter(
                        x=list(range(len(predictions))), y=predictions, name=preds_col
                    )
                )
                fig.add_trace(
                    go.Scatter(x=list(range(len(actuals))), y=actuals, name=actual_col)
                )
                fig.update_layout(
                    showlegend=True, xaxis_title="time", yaxis_title="value"
                )
                st.write(fig)

    def feature_histogram(self, dir_path, dataslice_schemas):
        """
        Distributin generation, includes:
        1. categorical variable distribution if any
        2. Prediction vs actuals distribution for all the dataslice available
        3. Numerical features distribution
        """
        ## Getting the whylogs histogram
        profile_summary = self.mlfoundry_data.get_whylogs_summary(
            st.session_state[ID_SESSION_STATE_MODEL_VIEW]
        )
        if profile_summary is None:
            st.warning(
                f"Stats doesn't exist for {st.session_state[ID_SESSION_STATE_MODEL_VIEW]}"
            )
            return
        summary = profile_summary["summary"]
        summary = summary.fillna("null")
        features = list(summary["column"])
        summary_hist = profile_summary["hist"]
        st.header("Feature Histogram")
        ### Actuals vs predictions
        model_type = self.mlfoundry_data.get_model_type(
            st.session_state[ID_SESSION_STATE_MODEL_VIEW]
        )
        if model_type in [
            MULTICLASS_CLASSIFICATION_MODEL_TYPE,
            BINARY_CLASSIFICATION_MODEL_TYPE,
        ]:
            self.actual_vs_preds_classification(dir_path, dataslice_schemas)
        elif model_type == REGRESSION_MODEL_TYPE:
            self.actual_vs_preds_regression(dir_path, dataslice_schemas)
        elif model_type == TIMESERIES_MODEL_TYPE:
            self.actual_vs_prediction_time_series(dir_path, dataslice_schemas)

        ### Catergorical feature distribution
        numerical_features = self.categorical_feature_distribution(
            dir_path, features, dataslice_schemas
        )

        ### Numerical feature distribution - from whylogs
        self.numerical_feature_distribution(numerical_features, summary_hist)

    def start_tab(self, project_name, id, run_ids):

        result = self.get_schema_shap()
        if result is None:
            return

        dir_path, dataslice_schemas, dataslice_shap = result

        self.shap_computation(dataslice_schemas, dataslice_shap)

        self.feature_histogram(dir_path, dataslice_schemas)



import os

import click

from mlfoundry_ui.constants import MLF_FOLDER_NAME, MLRUNS_FOLDER_NAME


def create_mlfoundry_ui_cli():
    """Generates CLI by combining all subcommands into a main CLI and returns in
    Returns:
        function: main CLI functions will all added sub-commands
    """
    _cli = mlfoundry_ui_cli()
    _cli.add_command(start_dashboard)
    return _cli


@click.group()
@click.version_option("1.0.0")
def mlfoundry_ui_cli():
    """MlFoundry UI CLI"""


@mlfoundry_ui_cli.command(
    help="Generate MLFoundry Dashboard",
    short_help="Generate MLFoundry Dashboard",
)
@click.option(
    "-p",
    "--path",
    type=click.Path(exists=True, dir_okay=True, readable=True),
    default=os.path.abspath("."),
)
def start_dashboard(path: str):
    abs_path = os.path.abspath(path)
    if not os.path.exists(os.path.join(abs_path, MLF_FOLDER_NAME)):
        raise Exception(
            f"Please provide a valid path of directory containing {MLF_FOLDER_NAME} directory. "
            f"{abs_path} does not contain {MLF_FOLDER_NAME}."
        )
    mlruns_path = os.path.join(abs_path, MLRUNS_FOLDER_NAME)

    import mlfoundry_ui as mlf_ui

    mlf_ui_main_file_path = os.path.join(
        os.path.dirname(mlf_ui.__file__), "webapp", "main_app.py"
    )
    os.system(f"streamlit run {mlf_ui_main_file_path} -- {mlruns_path}")



import os
import pickle

import numpy as np
import plotly.express as px
import plotly.figure_factory as ff
import plotly.graph_objects as go
import streamlit as st
from sklearn.metrics import auc

from mlfoundry_ui.webapp.dashboard_constants import *


class ModelHealth:
    def __init__(self, mlfoundry_data) -> None:
        self.mlfoundry_data = mlfoundry_data

    def __metric_name_helper(self, metric_name):
        """
        Given the metric name as stored in S3 returns str in Capitalised and preprocessed form
        Example:
        "mean_absolute_error_train" will be preprocessed and returned as "mean_absolute_error"
        """
        for metric_key, metric_value in METRIC_DICT.items():
            if metric_key in metric_name:
                return metric_value
        return metric_name

    def __roc_helper(self, model_type, multi_dim_metrics):
        fpr = multi_dim_metrics[MULTIDIM_METRICS_ROC][MULTIDIM_METRICS_FPR]
        tpr = multi_dim_metrics[MULTIDIM_METRICS_ROC][MULTIDIM_METRICS_TPR]

        if model_type == BINARY_CLASSIFICATION_MODEL_TYPE:
            fig = px.area(
                x=fpr,
                y=tpr,
                title=f"(AUC={auc(tpr, fpr):.4f})",
                labels=dict(x="False Positive Rate", y="True Positive Rate"),
            )
            fig.add_shape(type="line", line=dict(dash="dash"), x0=0, x1=1, y0=0, y1=1)

        elif model_type == MULTICLASS_CLASSIFICATION_MODEL_TYPE:
            fig = go.Figure()
            fig.add_shape(type="line", line=dict(dash="dash"), x0=0, x1=1, y0=0, y1=1)
            for i in range(len(fpr)):
                name = f"Class {i} (AUC={auc(tpr[i], fpr[i]):.2f})"
                fig.add_trace(go.Scatter(x=fpr[i], y=tpr[i], name=name, mode="lines"))
        # TODO (nikunjbjj): This fig can be referenced before assignment. Need to handle it cleanly.
        return fig

    def __pr_helper(self, model_type, multi_dim_metrics):
        precision = multi_dim_metrics[MULTIDIM_METRICS_PRECISION_RECALL][
            MULTIDIM_METRICS_PRECISION
        ]
        recall = multi_dim_metrics[MULTIDIM_METRICS_PRECISION_RECALL][
            MULTIDIM_METRICS_RECALL
        ]

        if model_type == BINARY_CLASSIFICATION_MODEL_TYPE:
            fig = px.area(x=recall, y=precision, labels=dict(x="Recall", y="Precision"))
            fig.add_shape(type="line", line=dict(dash="dash"), x0=0, x1=1, y0=1, y1=0)

        elif model_type == MULTICLASS_CLASSIFICATION_MODEL_TYPE:
            fig = go.Figure()
            fig.add_shape(type="line", line=dict(dash="dash"), x0=0, x1=1, y0=1, y1=0)
            for i in range(len(precision)):
                name = f"Class {i}"
                fig.add_trace(
                    go.Scatter(x=recall[i], y=precision[i], name=name, mode="lines")
                )

        return fig

    def __roc_pr_helper(self, model_type, multi_dim_metric_dict):
        """Helper function to generate roc and pr curve

        Args:
            model_type (str)
            multi_dim_metrics (dict): multi_dim_metrics fetched from S3

        """
        for data_slice, multi_dim_metrics in multi_dim_metric_dict.items():
            figs = {}
            if MULTIDIM_METRICS_ROC in multi_dim_metrics.keys():
                figs["ROC"] = self.__roc_helper(model_type, multi_dim_metrics)

            if MULTIDIM_METRICS_PRECISION_RECALL in multi_dim_metrics.keys():
                figs["PR"] = self.__pr_helper(model_type, multi_dim_metrics)

            ## Plot the roc and pr curve for the specific data slice
            cols = list(st.columns(2))
            for i, (key, fig) in enumerate(figs.items()):
                cols[i].subheader(f"{key} Curve - {data_slice} data")
                cols[i].write(fig)

    def __confusion_matrix_helper(self):
        """
        Plots confusion matrix for all the dataslice

        Returns:
            multi_dim_metric_dict: multi_dim_metric corresponding to each dataslice
        """
        dir_path = self.mlfoundry_data.get_artifact(
            st.session_state[ID_SESSION_STATE_MODEL_VIEW], MULTIDIM_METRICS
        )
        multi_dim_metric_dict = {}
        confusion_matrix_figs = {}
        for file_name in os.listdir(dir_path):
            with open(os.path.join(dir_path, file_name), "rb") as f:
                multi_dim_metrics = pickle.load(f)

            for data_slice in DATA_SLICES:
                if data_slice not in file_name:
                    continue
                multi_dim_metric_dict[data_slice] = multi_dim_metrics
                matrix = multi_dim_metrics[MULTIDIM_METRICS_CONFUSION_MATRIX]
                matrix = np.flip(matrix, 0)
                fig = ff.create_annotated_heatmap(
                    matrix,
                    colorscale="Viridis",
                    x=[str(x) for x in list(range(len(matrix)))],
                    y=[str(x) for x in list(range(len(matrix) - 1, -1, -1))],
                )
                fig.update_layout(xaxis_title="Predictions", yaxis_title="Actuals")
                confusion_matrix_figs[data_slice] = fig

        cols = list(st.columns(2))
        if len(confusion_matrix_figs.keys()) > 2:
            cols += list(st.columns(2))
        for i, (data_slice, confusion_matrix) in enumerate(
            confusion_matrix_figs.items()
        ):
            with cols[i]:
                st.subheader(f"Confusion Matrix - {data_slice}")
                st.write(confusion_matrix)

        return multi_dim_metric_dict

    def user_generated_metrics(self, id_df_user):

        if id_df_user.empty:
            return
        ## metrics that are lists, example: r2_score = [0.5,0.8,0.9]
        id_df_user_list = id_df_user[
            id_df_user[METRIC_DF_VALUE_COL].apply(lambda x: isinstance(x, list))
        ]
        # metrics that are sigle numbers
        id_df_user = id_df_user[
            ~id_df_user[METRIC_DF_VALUE_COL].apply(lambda x: isinstance(x, list))
        ]
        id_df_user[METRIC_DF_VALUE_COL] = id_df_user[METRIC_DF_VALUE_COL].map(
            lambda x: round(x, 2)
        )

        if len(id_df_user) > 0 or len(id_df_user_list) > 0:
            st.subheader("User Generated Metrics")
        #### User Generated metrics that are single numbers
        if len(id_df_user) > 0:
            fig = go.Figure(
                data=[
                    go.Bar(
                        x=list(id_df_user[METRIC_DF_KEY_COL]),
                        y=list(id_df_user[METRIC_DF_VALUE_COL]),
                        text=list(id_df_user[METRIC_DF_VALUE_COL]),
                        textposition="auto",
                    )
                ]
            )
            fig.update_layout(showlegend=True)
            st.write(fig)
        # user generated metrics that are lists
        if len(id_df_user_list) > 0:
            fig = go.Figure()
            for index, row in id_df_user_list.iterrows():
                fig.add_trace(
                    go.Scatter(
                        x=list(range(len(row[METRIC_DF_KEY_COL]))),
                        y=row[METRIC_DF_VALUE_COL],
                        name=row[METRIC_DF_KEY_COL],
                    )
                )
            fig.update_layout(showlegend=True, yaxis_title="value")
            st.write(fig)

    def __auto_generated_metrics_helper(self, pre_computed_metrics_df):
        """
        This function returns a dictionary where keys are the dataslice and
        values are metrics dataframe corresponding to the dataslice

        """
        pre_computed_metrics = {}
        if pre_computed_metrics_df.empty:
            return pre_computed_metrics
        for data_slice in DATA_SLICES:
            data_slice_df = pre_computed_metrics_df[
                pre_computed_metrics_df[METRIC_DF_KEY_COL].str.endswith(data_slice)
            ]
            if data_slice_df.empty:
                continue
            pre_computed_metrics[data_slice] = data_slice_df
            pre_computed_metrics[data_slice][METRIC_DF_KEY_COL] = pre_computed_metrics[
                data_slice
            ][METRIC_DF_KEY_COL].map(self.__metric_name_helper)

        return pre_computed_metrics

    def auto_generated_metrics(self, id_df_pre_computed):

        if id_df_pre_computed.empty:
            return
        ## auto generated metrics that are lists, example: r2_score = [0.5,0.8,0.9]
        id_df_pre_computed_list = id_df_pre_computed[
            id_df_pre_computed[METRIC_DF_VALUE_COL].apply(lambda x: isinstance(x, list))
        ]
        # metrics that are sigle numbers
        id_df_pre_computed = id_df_pre_computed[
            ~id_df_pre_computed[METRIC_DF_VALUE_COL].apply(
                lambda x: isinstance(x, list)
            )
        ]
        if len(id_df_pre_computed) > 0:
            id_df_pre_computed[METRIC_DF_VALUE_COL] = id_df_pre_computed[
                METRIC_DF_VALUE_COL
            ].map(lambda x: round(x, 2))

        pre_computed_metrics = self.__auto_generated_metrics_helper(id_df_pre_computed)
        pre_computed_metrics_list = self.__auto_generated_metrics_helper(
            id_df_pre_computed_list
        )

        ### Auto generated metric that are single number
        if len(pre_computed_metrics) > 0 or len(pre_computed_metrics_list) > 0:
            st.subheader("Auto Generated Metrics")
        if len(pre_computed_metrics) > 0:
            fig = go.Figure()
            for data_slice, metrics_df in pre_computed_metrics.items():
                fig.add_trace(
                    go.Bar(
                        x=list(metrics_df[METRIC_DF_KEY_COL]),
                        y=list(metrics_df[METRIC_DF_VALUE_COL]),
                        text=list(metrics_df[METRIC_DF_VALUE_COL]),
                        name=data_slice,
                        textposition="auto",
                    )
                )
            fig.update_layout(barmode="group", showlegend=True, yaxis_title="value")
            st.write(fig)

        ## auto generated metrics that are lists, example: r2_score = [0.5,0.8,0.9]
        if len(pre_computed_metrics_list) > 0:
            fig = go.Figure()
            # Each row in the dataframe will contain list of metrics, so plotting them one by one
            for data_slice, metrics_df_list in pre_computed_metrics_list.items():
                for index, row in metrics_df_list.iterrows():
                    fig.add_trace(
                        go.Scatter(
                            x=list(range(len(row[METRIC_DF_KEY_COL]))),
                            y=row[METRIC_DF_VALUE_COL],
                            name=f"{row[METRIC_DF_KEY_COL]} -  {data_slice}",
                        )
                    )
            fig.update_layout(showlegend=True, yaxis_title="value", xaxis_title="step")
            st.write(fig)

    def pre_computed_user_generated_metrics(self, metrics_df):
        """
        Returns user generated and auto generated metrics
        """

        id_df = metrics_df[
            metrics_df[METRIC_DF_RUNID_COL]
            == st.session_state[ID_SESSION_STATE_MODEL_VIEW]
        ]

        if not len(id_df) > 0:
            return

        id_df_pre_computed = id_df[
            id_df[METRIC_DF_KEY_COL].str.startswith(PRECOMPUTED_METRIC)
        ]  ## Filtering precomputed metrics
        id_df_user = id_df[
            ~id_df[METRIC_DF_KEY_COL].str.startswith(PRECOMPUTED_METRIC)
        ]  # filtering user generated metrics

        self.user_generated_metrics(id_df_user)
        self.auto_generated_metrics(id_df_pre_computed)

    def start_tab(self, project_name, run_ids):
        """
        List of things shown;
        1. model type
        2. confusion matrix for each dataslice
        3. roc and pr curve for each data slice
        4. auto computed and user generated metrics
        """

        metrics_df = self.mlfoundry_data.get_metrics(run_ids)
        if metrics_df is None:
            st.warning("No Run exists for the Project")
            return

        if ID_SESSION_STATE_MODEL_VIEW not in st.session_state:
            st.warning("Please specify a Run ID on the side bar")
            return

        model_type = self.mlfoundry_data.get_model_type(
            st.session_state[ID_SESSION_STATE_MODEL_VIEW]
        )
        st.header("Model metrics")

        if model_type in [
            MULTICLASS_CLASSIFICATION_MODEL_TYPE,
            BINARY_CLASSIFICATION_MODEL_TYPE,
        ]:
            #### CONFUSION MATRIX COMPUTATION
            multi_dim_metric_dict = self.__confusion_matrix_helper()
            #### ROC and PR Curve
            self.__roc_pr_helper(model_type, multi_dim_metric_dict)

        ## User and precomputed metrics
        self.pre_computed_user_generated_metrics(metrics_df)



import enum

METRIC_DICT = {
    "cohen_kappa": "Cohen Kappa Score",
    "f1_score": "F1 Score",
    "accuracy": "Accuracy",
    "r2_score": "R2 Score",
    "root_mean_squared_error": "Root Mean Squared Error",
    "mean_squared_error": "Mean Squared Error",
    "mean_absolute_error": "Mean Absolute Error",
    "log_loss": "Log Loss",
    "explained_variance_score": "Variance Score",
}

DATA_SLICES = ["train", "test", "validate", "prediction"]

COLORS = [
    "aliceblue",
    "antiquewhite",
    "aqua",
    "aquamarine",
    "azure",
    "beige",
    "bisque",
    "blue",
    "blueviolet",
    "brown",
    "burlywood",
    "cadetblue",
    "chartreuse",
    "chocolate",
    "coral",
    "cornflowerblue",
    "cornsilk",
    "crimson",
    "cyan",
    "darkblue",
    "darkcyan",
    "darkgoldenrod",
    "darkgray",
    "darkgrey",
    "darkgreen",
    "darkkhaki",
    "darkmagenta",
    "darkolivegreen",
    "darkorange",
    "darkorchid",
    "darkred",
    "darksalmon",
    "darkseagreen",
    "darkslateblue",
    "darkturquoise",
    "darkviolet",
    "deeppink",
    "deepskyblue",
]

BINARY_CLASSIFICATION_MODEL_TYPE = "binary_classification"
MULTICLASS_CLASSIFICATION_MODEL_TYPE = "multiclass_classification"
REGRESSION_MODEL_TYPE = "regression"
TIMESERIES_MODEL_TYPE = "timeseries"

ID_SESSION_STATE_MODEL_VIEW = "id_model"
RUN_NAME_SESSION_STATE_MODEL_VIEW = "run_name_model"
PROJECT_SESSION_STATE_MODEL_VIEW = "project_name_model"
PROJECT_SESSION_STATE_MODEL_COMPARISON = "project_name_comparison"
ACTUAL_PREDICTION_REGRESSION = "actuals_predictions_counts"

MODEL_TYPE_TAG = "modelType"
RUN_NAME_TAG = "run_name"
METRIC_DF_KEY_COL = "key"
METRIC_DF_VALUE_COL = "value"
METRIC_DF_RUNID_COL = "run_id"
METRIC_DF_RUN_NAME_COL = "run_name"

SCHEMA_FILE_NAME = "schema"
SHAP_VALUE_FILE_NAME = "shap"
UNIQUE_COUNTS_FILE_NAME = "unique_count"
TIME_SERIES_ACTUAL_PREDS_FILE_NAME = "actuals_prediction"
WHYLOGS_ARTIFACT_DIR = "whylogs"
WHYLOGS_BINEDGES = "bin_edges"
WHYLOGS_COUNTS = "counts"

PRECOMPUTED_METRIC = "pre_computed"
MULTIDIM_METRICS = "multi_dimensional_metrics"
MULTIDIM_METRICS_PRECISION_RECALL = "precision_recall_curve"
MULTIDIM_METRICS_PRECISION = "precision"
MULTIDIM_METRICS_RECALL = "recall"
MULTIDIM_METRICS_ROC = "roc_curve"
MULTIDIM_METRICS_FPR = "fpr"
MULTIDIM_METRICS_TPR = "tpr"
MULTIDIM_METRICS_CONFUSION_MATRIX = "confusion_matrix"


class Pages(enum.Enum):
    SINGLE_RUN_VIEW = "Single Run View"
    RUN_COMPARISON = "Run Comparison"



import os

MLF_FOLDER_NAME = "mlf"
MLRUNS_FOLDER = "mlruns"
MLRUNS_FOLDER_NAME = os.path.join(MLF_FOLDER_NAME, MLRUNS_FOLDER)

MLFOUNDRY_DOCUMENTATION_LINK = "https://truefoundry.gitbook.io/mlfoundry/"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mlfoundry_ui',
 'mlfoundry_ui.cli',
 'mlfoundry_ui.webapp',
 'mlfoundry_ui.webapp.model_comparison',
 'mlfoundry_ui.webapp.model_view']

package_data = \
{'': ['*']}

install_requires = \
['hydralit>=1.0.10,<2.0.0',
 'matplotlib>=3.0.3',
 'mlflow>=1.14',
 'numpy>=1.17.0',
 'pandas>=1.0.0',
 'plotly>=5.5.0,<6.0.0',
 'streamlit>=1.3.0,<2.0.0',
 'whylogs>=0.6.15']

entry_points = \
{'console_scripts': ['mlfoundry_ui = '
                     'mlfoundry_ui.cli.cli_interface:create_mlfoundry_ui_cli']}

setup_kwargs = {
    'name': 'mlfoundry-ui',
    'version': '0.0.9',
    'description': 'Dashboard code for mlfoundry',
    'long_description': '# mlfoundry-ui\n',
    'author': 'Abhishek Choudhary',
    'author_email': 'abhichoudhary06@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/truefoundry/mlfoundry-ui',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<3.10',
}


setup(**setup_kwargs)
